@@ -499,7 +499,6 @@ gst_asf_mux_get_content_description_tags (GstAsfMux * asfmux,
     GstAsfTags * asftags)
 {
   const GstTagList *tags;
-  GstTagList *taglist = NULL;
 
   tags = gst_tag_setter_get_tag_list (GST_TAG_SETTER (asfmux));
   if (tags && !gst_tag_list_is_empty (tags)) {
@@ -509,7 +508,6 @@ gst_asf_mux_get_content_description_tags (GstAsfMux * asfmux,
     asftags->tags = gst_tag_list_new ();
     asftags->cont_desc_size = 0;
     asftags->ext_cont_desc_size = 0;
-    taglist = asftags->tags;
 
     GST_DEBUG_OBJECT (asfmux, "Processing tags");
     gst_tag_list_foreach (tags, content_description_calc_size_for_tag, asftags);
@@ -919,7 +917,7 @@ gst_asf_mux_write_string_with_size (GstAsfMux * asfmux,
   if (error) {
     GST_WARNING_OBJECT (asfmux, "Error converting string "
         "to UTF-16: %s - %s", str, error->message);
-    g_free (error);
+    g_error_free (error);
     memset (str_buf, 0, str_size);
   } else {
     /* HACK: g_convert seems to add only a single byte null char to
