@@ -113,7 +113,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#  include "config.h"
+#include "config.h"
 #endif
 
 #include <stdlib.h>
@@ -122,10 +122,7 @@
 
 #include <glib.h>
 
-#ifndef M_PI
-# define M_PI           3.14159265358979323846  /* pi */
-#endif
-
+#include "gstdtmfcommon.h"
 
 #include "gstdtmfsrc.h"
 
@@ -136,12 +133,6 @@
 #define DEFAULT_SAMPLE_RATE      8000
 #define SAMPLE_SIZE              16
 #define CHANNELS                 1
-#define MIN_EVENT                0
-#define MAX_EVENT                16
-#define MIN_VOLUME               0
-#define MAX_VOLUME               36
-#define MIN_INTER_DIGIT_INTERVAL 100
-#define MIN_PULSE_DURATION       250
 #define MIN_DUTY_CYCLE           (MIN_INTER_DIGIT_INTERVAL + MIN_PULSE_DURATION)
 
 
@@ -223,6 +214,7 @@ static void gst_dtmf_src_set_property (GObject * object, guint prop_id,
 static void gst_dtmf_src_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec);
 static gboolean gst_dtmf_src_handle_event (GstBaseSrc * src, GstEvent * event);
+static gboolean gst_dtmf_src_send_event (GstElement * src, GstEvent * event);
 static GstStateChangeReturn gst_dtmf_src_change_state (GstElement * element,
     GstStateChange transition);
 static GstFlowReturn gst_dtmf_src_create (GstBaseSrc * basesrc,
@@ -264,17 +256,19 @@ gst_dtmf_src_class_init (GstDTMFSrcClass * klass)
   gstelement_class = GST_ELEMENT_CLASS (klass);
 
 
-  gobject_class->finalize = GST_DEBUG_FUNCPTR (gst_dtmf_src_finalize);
-  gobject_class->set_property = GST_DEBUG_FUNCPTR (gst_dtmf_src_set_property);
-  gobject_class->get_property = GST_DEBUG_FUNCPTR (gst_dtmf_src_get_property);
+  gobject_class->finalize = gst_dtmf_src_finalize;
+  gobject_class->set_property = gst_dtmf_src_set_property;
+  gobject_class->get_property = gst_dtmf_src_get_property;
 
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_INTERVAL,
       g_param_spec_uint ("interval", "Interval between tone packets",
           "Interval in ms between two tone packets", MIN_PACKET_INTERVAL,
-          MAX_PACKET_INTERVAL, DEFAULT_PACKET_INTERVAL, G_PARAM_READWRITE));
+          MAX_PACKET_INTERVAL, DEFAULT_PACKET_INTERVAL,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   gstelement_class->change_state =
       GST_DEBUG_FUNCPTR (gst_dtmf_src_change_state);
+  gstelement_class->send_event = GST_DEBUG_FUNCPTR (gst_dtmf_src_send_event);
   gstbasesrc_class->unlock = GST_DEBUG_FUNCPTR (gst_dtmf_src_unlock);
   gstbasesrc_class->unlock_stop = GST_DEBUG_FUNCPTR (gst_dtmf_src_unlock_stop);
 
@@ -403,6 +397,17 @@ gst_dtmf_src_handle_event (GstBaseSrc * src, GstEvent * event)
   return result;
 }
 
+
+static gboolean
+gst_dtmf_src_send_event (GstElement * element, GstEvent * event)
+{
+
+  if (gst_dtmf_src_handle_event (GST_BASE_SRC (element), event))
+    return TRUE;
+
+  return GST_ELEMENT_CLASS (parent_class)->send_event (element, event);
+}
+
 static void
 gst_dtmf_src_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec)
@@ -443,17 +448,11 @@ static void
 gst_dtmf_prepare_timestamps (GstDTMFSrc * dtmfsrc)
 {
   GstClock *clock;
-  GstClockTime base_time;
-
-  base_time = gst_element_get_base_time (GST_ELEMENT (dtmfsrc));
 
   clock = gst_element_get_clock (GST_ELEMENT (dtmfsrc));
   if (clock != NULL) {
-#ifdef MAEMO_BROKEN
-    dtmfsrc->timestamp = gst_clock_get_time (clock);
-#else
-    dtmfsrc->timestamp = gst_clock_get_time (clock) - base_time;
-#endif
+    dtmfsrc->timestamp = gst_clock_get_time (clock)
+        - gst_element_get_base_time (GST_ELEMENT (dtmfsrc));
     gst_object_unref (clock);
   } else {
     gchar *dtmf_name = gst_element_get_name (dtmfsrc);
@@ -682,12 +681,8 @@ gst_dtmf_src_create (GstBaseSrc * basesrc, guint64 offset,
 
   clock = gst_element_get_clock (GST_ELEMENT (basesrc));
 
-#ifdef MAEMO_BROKEN
-  clockid = gst_clock_new_single_shot_id (clock, dtmfsrc->timestamp);
-#else
   clockid = gst_clock_new_single_shot_id (clock, dtmfsrc->timestamp +
       gst_element_get_base_time (GST_ELEMENT (dtmfsrc)));
-#endif
   gst_object_unref (clock);
 
   GST_OBJECT_LOCK (dtmfsrc);
