@@ -723,7 +723,7 @@ tsmux_write_stream_packet (TsMux * mux, TsMuxStream * stream)
     /* Need to decide whether to write a new PCR in this packet */
     if (stream->last_pcr == -1 ||
         (cur_pcr - stream->last_pcr >
-            (TSMUX_CLOCK_FREQ / TSMUX_DEFAULT_PCR_FREQ))) {
+            (TSMUX_SYS_CLOCK_FREQ / TSMUX_DEFAULT_PCR_FREQ))) {
 
       stream->pi.flags |=
           TSMUX_PACKET_FLAG_ADAPTATION | TSMUX_PACKET_FLAG_WRITE_PCR;
@@ -767,8 +767,10 @@ tsmux_write_stream_packet (TsMux * mux, TsMuxStream * stream)
     }
   }
 
-  pi->stream_avail = tsmux_stream_bytes_avail (stream);
   pi->packet_start_unit_indicator = tsmux_stream_at_pes_start (stream);
+  if (pi->packet_start_unit_indicator)
+    tsmux_stream_initialize_pes_packet (stream);
+  pi->stream_avail = tsmux_stream_bytes_avail (stream);
 
   if (!tsmux_write_ts_header (mux->packet_buf, pi, &payload_len, &payload_offs))
     return FALSE;
