@@ -280,7 +280,7 @@ static gboolean
 gst_y4m_dec_parse_header (GstY4mDec * y4mdec, char *header)
 {
   char *end;
-  int format = -1;
+  int format = 420;
   int interlaced_char = 0;
 
   if (memcmp (header, "YUV4MPEG2 ", 10) != 0) {
@@ -744,4 +744,5 @@ plugin_init (GstPlugin * plugin)
 GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
     GST_VERSION_MINOR,
     "y4mdec",
-    "FIXME", plugin_init, VERSION, "LGPL", PACKAGE_NAME, GST_PACKAGE_ORIGIN)
+    "Demuxes/decodes YUV4MPEG streams",
+    plugin_init, VERSION, "LGPL", PACKAGE_NAME, GST_PACKAGE_ORIGIN)
