@@ -39,7 +39,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#  include <config.h>
+#include <config.h>
 #endif
 
 #include <gst/gst.h>
@@ -248,6 +248,9 @@ gst_vdp_vpp_get_required_pictures (GstVdpVideoPostProcess * vpp)
     case GST_VDP_DEINTERLACE_METHOD_TEMPORAL_SPATIAL:
       ret = 2;
       break;
+    default:
+      g_assert_not_reached ();
+      break;
   }
 
   return ret;
@@ -501,7 +504,6 @@ gst_vdp_vpp_sink_setcaps (GstPad * pad, GstCaps * caps)
     }
   }
 
-
   structure = gst_caps_get_structure (video_caps, 0);
   if (!gst_structure_get_int (structure, "width", &vpp->width) ||
       !gst_structure_get_int (structure, "height", &vpp->height) ||
@@ -522,26 +524,11 @@ gst_vdp_vpp_sink_setcaps (GstPad * pad, GstCaps * caps)
   } else
     vpp->got_par = FALSE;
 
-  if (gst_vdp_vpp_is_interlaced (vpp)) {
-    gint fps_n, fps_d;
-
-    if (gst_structure_get_fraction (structure, "framerate", &fps_n, &fps_d)) {
-      gst_fraction_double (&fps_n, &fps_d);
-      gst_structure_set (structure, "framerate", GST_TYPE_FRACTION, fps_n,
-          fps_d, NULL);
-      vpp->field_duration = gst_util_uint64_scale (GST_SECOND, fps_d, fps_n);
-    }
-
-    gst_structure_remove_field (structure, "interlaced");
-  }
-
   allowed_caps = gst_pad_get_allowed_caps (vpp->srcpad);
   if (G_UNLIKELY (!allowed_caps))
-    goto allowed_caps_error;
-  if (G_UNLIKELY (gst_caps_is_empty (allowed_caps))) {
-    gst_caps_unref (allowed_caps);
-    goto allowed_caps_error;
-  }
+    goto null_allowed_caps;
+  if (G_UNLIKELY (gst_caps_is_empty (allowed_caps)))
+    goto empty_allowed_caps;
   GST_DEBUG ("allowed_caps: %" GST_PTR_FORMAT, allowed_caps);
 
   output_caps = gst_vdp_video_to_output_caps (video_caps);
@@ -554,6 +541,20 @@ gst_vdp_vpp_sink_setcaps (GstPad * pad, GstCaps * caps)
 
   gst_pad_fixate_caps (vpp->srcpad, src_caps);
 
+
+  if (gst_vdp_vpp_is_interlaced (vpp)) {
+    gint fps_n, fps_d;
+
+    if (gst_structure_get_fraction (structure, "framerate", &fps_n, &fps_d)) {
+      gst_fraction_double (&fps_n, &fps_d);
+      gst_caps_set_simple (src_caps, "framerate", GST_TYPE_FRACTION, fps_n,
+          fps_d, NULL);
+      vpp->field_duration = gst_util_uint64_scale (GST_SECOND, fps_d, fps_n);
+    }
+
+    gst_caps_set_simple (src_caps, "interlaced", G_TYPE_BOOLEAN, FALSE, NULL);
+  }
+
   GST_DEBUG ("src_caps: %" GST_PTR_FORMAT, src_caps);
 
   res = gst_pad_set_caps (vpp->srcpad, src_caps);
@@ -566,8 +567,14 @@ done:
 
   return res;
 
-allowed_caps_error:
-  GST_ERROR_OBJECT (vpp, "Got invalid allowed caps");
+null_allowed_caps:
+  GST_ERROR_OBJECT (vpp, "Got null from gst_pad_get_allowed_caps");
+  goto done;
+
+empty_allowed_caps:
+  GST_ERROR_OBJECT (vpp, "Got EMPTY caps from gst_pad_get_allowed_caps");
+
+  gst_caps_unref (allowed_caps);
   goto done;
 
 not_negotiated:
