@@ -65,8 +65,7 @@ gtk_widget_get_allocation (GtkWidget * w, GtkAllocation * a)
 
 #define PREVIEW_TIME_MS (2 * 1000)
 #define N_BURST_IMAGES 10
-#define DEFAULT_UI_FILE "gst-camera.ui"
-#define SHARED_UI_FILE CAMERA_APPS_UIDIR"/"DEFAULT_UI_FILE
+#define UI_FILE CAMERA_APPS_UIDIR G_DIR_SEPARATOR_S "gst-camera.ui"
 
 /* Names of default elements */
 #define CAMERA_APP_VIDEOSRC "v4l2src"
@@ -275,8 +274,12 @@ my_bus_sync_callback (GstBus * bus, GstMessage * message, gpointer data)
     return GST_BUS_PASS;
 
   /* FIXME: make sure to get XID in main thread */
-  gst_x_overlay_set_xwindow_id (GST_X_OVERLAY (message->src),
+  gst_x_overlay_set_window_handle (GST_X_OVERLAY (message->src),
+#if GTK_CHECK_VERSION (2, 91, 6)
+      GDK_WINDOW_XID (gtk_widget_get_window (ui_drawing));
+#else
       GDK_WINDOW_XWINDOW (gtk_widget_get_window (ui_drawing)));
+#endif
 
   gst_message_unref (message);
   return GST_BUS_DROP;
@@ -545,6 +548,7 @@ me_gst_setup_pipeline (const gchar * imagepost, const gchar * videopost)
   /* set properties */
   g_object_set (gst_camera_bin, "filename", filename->str, NULL);
   g_object_set (gst_camera_bin, "preview-caps", preview_caps, NULL);
+  g_object_set (gst_camera_bin, "flags", 0xdf, NULL);
   gst_caps_unref (preview_caps);
 
   gst_videosrc = gst_element_factory_make (CAMERA_APP_VIDEOSRC, NULL);
@@ -757,7 +761,7 @@ set_metadata (void)
   /* for more information about image metadata tags, see:
    * http://webcvs.freedesktop.org/gstreamer/gst-plugins-bad/tests/icles/metadata_editor.c
    * and for the mapping:
-   * http://webcvs.freedesktop.org/gstreamer/gst-plugins-bad/ext/metadata/metadata_mapping.htm?view=co 
+   * http://webcvs.freedesktop.org/gstreamer/gst-plugins-bad/ext/metadata/metadata_mapping.htm?view=co
    */
 
   GstTagSetter *setter = GST_TAG_SETTER (gst_camera_bin);
@@ -819,11 +823,11 @@ on_buttonPause_clicked (GtkButton * button, gpointer user_data)
     case CAP_STATE_IMAGE:
       if (g_str_equal (gtk_button_get_label (ui_bnt_pause), "Focus")) {
         /* Start autofocus */
-        gst_photography_set_autofocus (GST_PHOTOGRAPHY (gst_camera_bin), TRUE);
+        gst_photography_set_autofocus (GST_PHOTOGRAPHY (gst_videosrc), TRUE);
         gtk_button_set_label (ui_bnt_pause, "Cancel Focus");
       } else {
         /* Cancel autofocus */
-        gst_photography_set_autofocus (GST_PHOTOGRAPHY (gst_camera_bin), FALSE);
+        gst_photography_set_autofocus (GST_PHOTOGRAPHY (gst_videosrc), FALSE);
         gtk_button_set_label (ui_bnt_pause, "Focus");
       }
       break;
@@ -1034,6 +1038,9 @@ on_color_control_value_changed (GtkRange * range, gpointer user_data)
   gst_color_balance_set_value (balance, channel, val);
 }
 
+#ifndef GDK_KEY_F11
+#define GDK_KEY_F11 GDK_F11
+#endif
 
 gboolean
 on_key_released (GtkWidget * widget, GdkEventKey * event, gpointer user_data)
@@ -1041,9 +1048,9 @@ on_key_released (GtkWidget * widget, GdkEventKey * event, gpointer user_data)
   g_return_val_if_fail (event != NULL, FALSE);
 
   switch (event->keyval) {
-    case GDK_F11:
+    case GDK_KEY_F11:
 #ifdef HAVE_GST_PHOTO_IFACE_H
-      gst_photography_set_autofocus (GST_PHOTOGRAPHY (gst_camera_bin), FALSE);
+      gst_photography_set_autofocus (GST_PHOTOGRAPHY (gst_videosrc), FALSE);
 #endif
       break;
     default:
@@ -1059,9 +1066,9 @@ on_key_pressed (GtkWidget * widget, GdkEventKey * event, gpointer user_data)
   g_return_val_if_fail (event != NULL, FALSE);
 
   switch (event->keyval) {
-    case GDK_F11:
+    case GDK_KEY_F11:
 #ifdef HAVE_GST_PHOTO_IFACE_H
-      gst_photography_set_autofocus (GST_PHOTOGRAPHY (gst_camera_bin), TRUE);
+      gst_photography_set_autofocus (GST_PHOTOGRAPHY (gst_videosrc), TRUE);
 #endif
       break;
     case 0x0:
@@ -1097,6 +1104,7 @@ format_value_callback (GtkScale * scale, gdouble value, gpointer user_data)
 static gint
 create_menu_items_from_structure (GstStructure * structure)
 {
+  GtkListStore *store;
   const GValue *framerate_list = NULL;
   const gchar *structure_name;
   GString *item_str = NULL;
@@ -1151,8 +1159,10 @@ create_menu_items_from_structure (GstStructure * structure)
       goto range_found;
     }
 
+    store = GTK_LIST_STORE (gtk_combo_box_get_model (ui_cbbox_resolution));
     for (j = 0; j < num_framerates; j++) {
       GstCaps *video_caps;
+      GtkTreeIter iter;
 
       if (framerate_list) {
         const GValue *item = gst_value_list_get_value (framerate_list, j);
@@ -1163,7 +1173,8 @@ create_menu_items_from_structure (GstStructure * structure)
       g_string_append_printf (item_str, " (%" GST_FOURCC_FORMAT ")",
           GST_FOURCC_ARGS (fourcc));
       g_string_append_printf (item_str, ", %dx%d at %d/%d", w, h, n, d);
-      gtk_combo_box_append_text (ui_cbbox_resolution, item_str->str);
+      gtk_list_store_append (store, &iter);
+      gtk_list_store_set (store, &iter, 0, item_str->str, -1);
 
       video_caps =
           gst_caps_new_simple (structure_name, "format", GST_TYPE_FOURCC,
@@ -1337,31 +1348,31 @@ photo_menuitem_toggled_cb (GtkRadioMenuItem * menuitem, gpointer user_data)
       GstWhiteBalanceMode mode;
       ret =
           gst_photography_set_white_balance_mode (GST_PHOTOGRAPHY
-          (gst_camera_bin), val->value);
-      gst_photography_get_white_balance_mode (GST_PHOTOGRAPHY (gst_camera_bin),
+          (gst_videosrc), val->value);
+      gst_photography_get_white_balance_mode (GST_PHOTOGRAPHY (gst_videosrc),
           &mode);
       set_value = (gint) mode;
     } else if (etype == GST_TYPE_SCENE_MODE) {
       GstSceneMode mode;
       ret =
-          gst_photography_set_scene_mode (GST_PHOTOGRAPHY (gst_camera_bin),
+          gst_photography_set_scene_mode (GST_PHOTOGRAPHY (gst_videosrc),
           val->value);
-      gst_photography_get_scene_mode (GST_PHOTOGRAPHY (gst_camera_bin), &mode);
+      gst_photography_get_scene_mode (GST_PHOTOGRAPHY (gst_videosrc), &mode);
       set_value = (gint) mode;
     } else if (etype == GST_TYPE_COLOUR_TONE_MODE) {
       GstColourToneMode mode;
       ret =
           gst_photography_set_colour_tone_mode (GST_PHOTOGRAPHY
-          (gst_camera_bin), val->value);
-      gst_photography_get_colour_tone_mode (GST_PHOTOGRAPHY (gst_camera_bin),
+          (gst_videosrc), val->value);
+      gst_photography_get_colour_tone_mode (GST_PHOTOGRAPHY (gst_videosrc),
           &mode);
       set_value = (gint) mode;
     } else if (etype == GST_TYPE_FLASH_MODE) {
       GstFlashMode mode;
       ret =
-          gst_photography_set_flash_mode (GST_PHOTOGRAPHY (gst_camera_bin),
+          gst_photography_set_flash_mode (GST_PHOTOGRAPHY (gst_videosrc),
           val->value);
-      gst_photography_get_flash_mode (GST_PHOTOGRAPHY (gst_camera_bin), &mode);
+      gst_photography_get_flash_mode (GST_PHOTOGRAPHY (gst_videosrc), &mode);
       set_value = (gint) mode;
     }
 
@@ -1388,11 +1399,10 @@ photo_iso_speed_toggled_cb (GtkRadioMenuItem * menuitem, gpointer user_data)
     if (!g_str_equal (name, "auto")) {
       sscanf (name, "%d", &val);
     }
-    if (!gst_photography_set_iso_speed (GST_PHOTOGRAPHY (gst_camera_bin), val)) {
+    if (!gst_photography_set_iso_speed (GST_PHOTOGRAPHY (gst_videosrc), val)) {
       g_print ("ISO speed (%d) setting failed\n", val);
     } else {
-      gst_photography_get_iso_speed (GST_PHOTOGRAPHY (gst_camera_bin),
-          &set_val);
+      gst_photography_get_iso_speed (GST_PHOTOGRAPHY (gst_videosrc), &set_val);
       if (val != set_val) {
         g_print ("ISO speed (%d) setting failed, got %d\n", val, set_val);
       }
@@ -1411,11 +1421,11 @@ photo_ev_comp_toggled_cb (GtkRadioMenuItem * menuitem, gpointer user_data)
   if (active) {
     name = gtk_widget_get_name (GTK_WIDGET (menuitem));
     sscanf (name, "%f", &val);
-    if (!gst_photography_set_ev_compensation (GST_PHOTOGRAPHY (gst_camera_bin),
+    if (!gst_photography_set_ev_compensation (GST_PHOTOGRAPHY (gst_videosrc),
             val)) {
       g_print ("EV compensation (%.1f) setting failed\n", val);
     } else {
-      gst_photography_get_ev_compensation (GST_PHOTOGRAPHY (gst_camera_bin),
+      gst_photography_get_ev_compensation (GST_PHOTOGRAPHY (gst_videosrc),
           &set_val);
       if (val != set_val) {
         g_print ("EV compensation (%.1f) setting failed, got %.1f\n", val,
@@ -1611,14 +1621,9 @@ static gboolean
 ui_create (void)
 {
   GError *error = NULL;
-  const gchar *uifile = DEFAULT_UI_FILE;
-
-  if (!g_file_test (uifile, G_FILE_TEST_EXISTS)) {
-    uifile = SHARED_UI_FILE;
-  }
 
   builder = gtk_builder_new ();
-  if (!gtk_builder_add_from_file (builder, uifile, &error)) {
+  if (!gtk_builder_add_from_file (builder, UI_FILE, &error)) {
     g_warning ("Couldn't load builder file: %s", error->message);
     g_error_free (error);
     goto done;
